package com.dingding.mid.listener;

import static com.dingding.mid.common.CommonConstants.BUSINESS_STATUS_4;
import static com.dingding.mid.common.CommonConstants.PROCESS_STATUS;
import static com.dingding.mid.common.WorkFlowConstants.PROCESS_PREFIX;

import javax.annotation.Resource;

import com.dingding.mid.entity.ProcessTemplates;
import com.dingding.mid.entity.WorkflowConfig;
import com.dingding.mid.mqtt.WorkFlowMessage;
import com.dingding.mid.service.ProcessTemplateService;
import com.dingding.mid.service.SendMessageService;
import com.dingding.mid.service.WorkflowConfigService;
import lombok.extern.slf4j.Slf4j;
import org.flowable.engine.HistoryService;
import org.flowable.engine.RepositoryService;
import org.flowable.engine.delegate.DelegateExecution;
import org.flowable.engine.delegate.ExecutionListener;
import org.flowable.engine.history.HistoricProcessInstance;
import org.springframework.stereotype.Component;

import java.util.Optional;

/**
 * @author LoveMyOrange
 * @create 2022-10-15 19:47
 */
@Component
@Slf4j
public class ProcessListener implements ExecutionListener {

    @Resource
    private HistoryService historyService;

    @Resource
    private ProcessTemplateService processTemplateService;

    @Resource
    private SendMessageService sendMessageService;

    @Resource
    private WorkflowConfigService workflowConfigService;

    @Override
    public void notify(DelegateExecution execution) {
        log.info("ProcessListener触发了");
        execution.setVariable(PROCESS_STATUS, BUSINESS_STATUS_4);
        String processInstanceId = execution.getProcessInstanceId();
        log.info("processInstanceId:{}", processInstanceId);

        // TODO: 2024/3/4 根据流程实例id 修改表单的数据 修改状态
        HistoricProcessInstance historicProcessInstance = historyService.createHistoricProcessInstanceQuery().processInstanceId(processInstanceId)
                .includeProcessVariables().singleResult();
        String processDefinitionKey = historicProcessInstance.getProcessDefinitionKey();
        ProcessTemplates processTemplates = processTemplateService.getById(processDefinitionKey.replace(PROCESS_PREFIX, ""));
        String tableConfigId = processTemplates.getTableConfigId();
        log.info("tableConfigId:{}", tableConfigId);
        log.info("ProcessVariables:{}",historicProcessInstance.getProcessVariables());
        if (tableConfigId != null) {
            //根据tableConfigId 查询需要修改的表单
            WorkflowConfig workflowConfig = workflowConfigService.getById(tableConfigId);
            //获取表名
            String tableName = workflowConfig.getTableName();
            //MQTT
            String command = "UPDATE-STATE-FINISH";
            WorkFlowMessage workFlowMessage = new WorkFlowMessage().setTableName(tableName).setProcessInstanceId(processInstanceId).setCommand(command);
            //发送表名和实例id
            String topic = "workflow/park/company";
            boolean publish = sendMessageService.publish(topic, workFlowMessage);
            if (publish) {
                log.info("发送成功");
            } else {
                log.info("发送失败");
            }
            // TODO: 2024/3/8 给发起人发送审核完成消息
        }
    }
}
