package com.dingding.mid.listener;

import cn.hutool.core.util.StrUtil;
import cn.hutool.json.JSONObject;
import cn.hutool.json.JSONUtil;
import com.dingding.mid.mqtt.CloudMessage;
import com.dingding.mid.service.SendMessageService;
import com.dingding.mid.utils.SpringContextHolder;
import liquibase.pro.packaged.R;
import lombok.extern.slf4j.Slf4j;
import org.flowable.engine.RuntimeService;
import org.flowable.engine.TaskService;
import org.flowable.engine.delegate.TaskListener;
import org.flowable.task.service.delegate.DelegateTask;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;

import java.time.LocalDateTime;

import static com.dingding.mid.common.CommonConstants.EXTEND_DATA;
import static com.dingding.mid.common.WorkFlowConstants.*;

/**
 * @author LoveMyOrange
 * @create 2022-10-15 14:51
 * @desc 精髓代码实现2 ,看下如何实现的审批人为空自动通过的逻辑
 */
@Component
@Slf4j
public class TaskCreatedListener implements TaskListener {
    @Resource
    private TaskService taskService;

    @Resource
    private SendMessageService sendMessageService;

    @Override
    public void notify(DelegateTask delegateTask) {
        log.info("TaskCreatedListener触发了"+delegateTask.toString());
            log.info(" 操作人:{}",delegateTask.getAssignee());
            log.info(" 执行id:{}",delegateTask.getExecutionId());
            log.info(" 实例id:{}",delegateTask.getProcessInstanceId());
            log.info(" 任务类别:{}",delegateTask.getCategory());
            log.info(" 变量信息:{}",delegateTask.getVariables());
            log.info(" 任务描述:{}",delegateTask.getDescription());
            log.info(" 事件名称:{}",delegateTask.getEventName());
            log.info(" 任务优先级:{}",delegateTask.getPriority());
            log.info(" 任务名称:{}",delegateTask.getName());
            log.info(" 当前任务事件的名称:{}",delegateTask.getEventName());
            log.info(" 扩展信息:{}",delegateTask.getVariable(EXTEND_DATA));

            if(DEFAULT_NULL_ASSIGNEE.equals(delegateTask.getAssignee())){
                Object autoRefuse = delegateTask.getVariable(AUTO_REFUSE_STR);
                log.info("autoRefuse :{} 审批人为空",autoRefuse);
                if(autoRefuse==null){
                    taskService.addComment(delegateTask.getId(),delegateTask.getProcessInstanceId(),OPINION_COMMENT,"审批人为空,自动通过");
                    taskService.complete(delegateTask.getId());
                } else{
                    taskService.addComment(delegateTask.getId(),delegateTask.getProcessInstanceId(),OPINION_COMMENT,"审批人为空,自动驳回");
                    RuntimeService runtimeService = SpringContextHolder.getBean(RuntimeService.class);
                    runtimeService.deleteProcessInstance(delegateTask.getProcessInstanceId(),"审批人为空,自动驳回");
                }
            } else {
                if ("审批人".equals(delegateTask.getName()) || "办理人".equals(delegateTask.getName()) || "审批人".equals(delegateTask.getName())) {
                    if ("create".equals(delegateTask.getEventName())) {
                        Object extendData = delegateTask.getVariable(EXTEND_DATA);
                        if (JSONUtil.isTypeJSON(extendData == null ? "" : extendData.toString())) {
                            log.info("EXTEND_DATA is json");
                            // 获取
                            JSONObject jsonObject = JSONUtil.parseObj(extendData);
                            String dataFrom = jsonObject.getStr("dataFrom");
                            String dataId = jsonObject.getStr("dataId");
                            if (StrUtil.isAllNotBlank(dataFrom,dataId)) {
                                // 发送消息
                                CloudMessage cloudMessage = new CloudMessage();
                                cloudMessage.setContent(String.format("您有新的%s待处理",delegateTask.getName()));
                                cloudMessage.setReceiveBy(delegateTask.getAssignee())
                                        .setCreateTime(LocalDateTime.now())
                                        .setTypeCode(dataFrom)
                                        .setOutId(delegateTask.getProcessInstanceId())
                                        .setExtend(extendData.toString())
                                        .setMessageLevel(1)
                                        .setDealStatus(0)
                                        .setMessageStatus(0)
                                        .setMessageTypeId(4)
                                        .setTypeName(dataFrom)
                                        .setTaskId(delegateTask.getId())
                                        .setTitle(delegateTask.getName() + "消息");
                                log.info(" 操作人:{}",delegateTask.getAssignee());
                                sendMessageService.publish("message/sys/workflow",cloudMessage);
                            } else {
                                log.info("EXTEND_DATA is not json");
                            }
                        }

                    }
                }
            }

    }
}
